@@ -5,17 +5,216 @@ use strict;
 use Carp();
 use Archive::Har::Entry::Cache::Request();
 
+our $VERSION = 0.07;
+
+sub new {
+	my ($class, $params) = @_;
+	my $self = {};
+	bless $self, $class;
+	if (defined $params) {
+		if (defined $params->{blocked}) {
+			$self->blocked($params->{blocked});
+		}
+		if (defined $params->{dns}) {
+			$self->dns($params->{dns});
+		}
+		if (defined $params->{connect}) {
+			$self->connect($params->{connect});
+		}
+		$self->send($params->{send});
+		$self->wait($params->{wait});
+		$self->receive($params->{receive});
+		if (defined $params->{ssl}) {
+			$self->ssl($params->{ssl});
+		}
+		if (defined $params->{comment}) {
+			$self->comment($params->{comment});
+		}
+		foreach my $key (sort { $a cmp $b } keys %{$params}) {
+			if ($key =~ /^_[[:alnum:]]+$/smx) { # private fields
+				if (defined $params->{$key}) {
+					$self->$key($params->{$key});
+				}
+			}
+		}
+	}
+	return $self;
+}
+
+sub blocked {
+	my ($self, $new) = @_;
+	my $old = $self->{blocked};
+	if (@_ > 1) {
+		if (defined $new) {
+			$self->{blocked} = $new;
+		} else {
+			$self->{blocked} = '-1';
+		}
+	}
+	if ((defined $old) && ($old eq '-1')) {
+		return;
+	} else {
+		return $old;
+	}
+}
+
+sub dns {
+	my ($self, $new) = @_;
+	my $old = $self->{dns};
+	if (@_ > 1) {
+		if (defined $new) {
+			$self->{dns} = $new;
+		} else {
+			$self->{dns} = '-1';
+		}
+	}
+	if ((defined $old) && ($old eq '-1')) {
+		return;
+	} else {
+		return $old;
+	}
+}
+
+sub connect {
+	my ($self, $new) = @_;
+	my $old = $self->{connect};
+	if (@_ > 1) {
+		if (defined $new) {
+			$self->{connect} = $new;
+		} else {
+			$self->{connect} = '-1';
+		}
+	}
+	if ((defined $old) && ($old eq '-1')) {
+		return;
+	} else {
+		return $old;
+	}
+}
+
+sub send {
+	my ($self, $new) = @_;
+	my $old = $self->{send};
+	if (@_ > 1) {
+		$self->{send} = $new;
+	}
+	return $old;
+}
+
+sub wait {
+	my ($self, $new) = @_;
+	my $old = $self->{wait};
+	if (@_ > 1) {
+		$self->{wait} = $new;
+	}
+	return $old;
+}
+
+sub receive {
+	my ($self, $new) = @_;
+	my $old = $self->{receive};
+	if (@_ > 1) {
+		$self->{receive} = $new;
+	}
+	return $old;
+}
+
+sub ssl {
+	my ($self, $new) = @_;
+	my $old = $self->{ssl};
+	if (@_ > 1) {
+		if (defined $new) {
+			$self->{ssl} = $new;
+		} else {
+			$self->{ssl} = '-1';
+		}
+	}
+	if ((defined $old) && ($old eq '-1')) {
+		return;
+	} else {
+		return $old;
+	}
+}
+
+sub comment {
+	my ($self, $new) = @_;
+	my $old = $self->{comment};
+	if (@_ > 1) {
+		$self->{comment} = $new;
+	}
+	return $old;
+}
+
+sub AUTOLOAD {
+	my ($self, $new) = @_;
+	my $type = ref $self or Carp::croak("$self is not an object");
+
+	my $name = $Archive::Har::Page::PageTimings::AUTOLOAD;
+	$name =~ s/.*://smx;   # strip fully-qualified portion
+
+	if ($name =~ /^_[[:alnum:]]+$/smx) { # private fields
+		my $old = $self->{$name};
+		if (@_ > 1) {
+			$self->{$name} = $new;
+		}
+		return $self->{$name};
+	} else {
+		Carp::croak("$name is not specified in the HAR 1.2 spec and does not start with an underscore");
+	}
+	return;
+}
+
+sub TO_JSON {
+	my ($self) = @_;
+	my $json = {};
+	if (defined $self->blocked()) {
+		$json->{blocked} = $self->blocked();
+	} else {
+		$json->{blocked} = -1;
+	}
+	if (defined $self->dns()) {
+		$json->{dns} = $self->dns();
+	} else {
+		$json->{dns} = -1;
+	}
+	if (defined $self->connect()) {
+		$json->{connect} = $self->connect();
+	} else {
+		$json->{connect} = -1;
+	}
+	$json->{send} = $self->send() || 0;
+	$json->{wait} = $self->wait() || 0;
+	$json->{receive} = $self->receive() || 0;
+	if (defined $self->ssl()) {
+		$json->{ssl} = $self->ssl();
+	} else {
+		$json->{ssl} = -1;
+	}
+	if (defined $self->comment()) {
+		$json->{comment} = $self->comment();
+	}
+	foreach my $key (sort { $a cmp $b } keys %{$self}) {
+		next if (!defined $self->{$key});
+		if ($key =~ /^_[[:alnum:]]+$/smx) { # private fields
+			$json->{$key} = $self->{$key};
+		}
+	}
+	
+	return $json;
+}
+
+sub DESTROY {
+}
+
+1;
+__END__
 =head1 NAME
 
 Archive::Har::Entry::Timings - Represents the timings for the individual phases during a request/response pair inside the HTTP Archive
 
 =head1 VERSION
 
-Version 0.06
-
-=cut
-
-our $VERSION = '0.06';
+Version 0.07
 
 =head1 SYNOPSIS
 
@@ -38,85 +237,18 @@ our $VERSION = '0.06';
 
 =head1 SUBROUTINES/METHODS
 
-=cut
-
-my (%_fields) = (
-			blocked => undef,
-			dns => undef,
-			connect => undef,
-			send => undef,
-			wait => undef,
-			receive => undef,
-			ssl => undef,
-			comment => undef,
-		);
-
 =head2 blocked
 
 returns the time in milliseconds spent waiting for a network connection.  Ihe function will return undef if it does not apply to the current request
 
-=cut
-
-sub blocked {
-	my ($self, $blocked) = @_;
-	my $old = $self->{blocked};
-	if ((defined $old) && ($old eq '-1')) {
-		$old = undef;
-	}
-	if (@_ > 1) {
-		if (defined $blocked) {
-			$self->{blocked} = $blocked;
-		} else {
-			$self->{blocked} = '-1';
-		}
-	}
-	return $old;
-}
-
 =head2 dns
 
 returns the time in milliseconds spent in DNS resolution of the host name.  Ihe function will return undef if it does not apply to the current request
 
-=cut
-
-sub dns {
-	my ($self, $dns) = @_;
-	my $old = $self->{dns};
-	if ((defined $old) && ($old eq '-1')) {
-		$old = undef;
-	}
-	if (@_ > 1) {
-		if (defined $dns) {
-			$self->{dns} = $dns;
-		} else {
-			$self->{dns} = '-1';
-		}
-	}
-	return $old;
-}
-
 =head2 connect
 
 returns the time in milliseconds spent making the TCP connection.  Ihe function will return undef if it does not apply to the current request
 
-=cut
-
-sub connect {
-	my ($self, $connect) = @_;
-	my $old = $self->{connect};
-	if ((defined $old) && ($old eq '-1')) {
-		$old = undef;
-	}
-	if (@_ > 1) {
-		if (defined $connect) {
-			$self->{connect} = $connect;
-		} else {
-			$self->{connect} = '-1';
-		}
-	}
-	return $old;
-}
-
 =head2 send
 
 returns the time in milliseconds spent sending the request to the server.
@@ -133,97 +265,7 @@ returns the time in milliseconds spent reading the response from the server.
 
 returns the time in milliseconds spent negotiating the SSL/TLS session.  Ihe function will return undef if it does not apply to the current request
 
-=cut
-
-sub ssl {
-	my ($self, $ssl) = @_;
-	my $old = $self->{ssl};
-	if ((defined $old) && ($old eq '-1')) {
-		$old = undef;
-	}
-	if (@_ > 1) {
-		if (defined $ssl) {
-			$self->{ssl} = $ssl;
-		} else {
-			$self->{ssl} = '-1';
-		}
-	}
-	return $old;
-}
-
 =head2 comment
 
 returns the comment about the page timings
 
-=cut
-
-sub new {
-	my ($class, $params) = @_;
-	my $self = {};
-	bless $self, $class;
-	foreach my $key (sort { $a cmp $b } keys %_fields) {
-		next unless (defined $params->{$key});
-		my $key_class = ref $params->{$key};
-		if ((defined $key_class) && ($key_class =~ /^JSON.*?Boolean$/smx)) {
-			$self->$key($params->{$key} ? 1 : 0);
-		} else {
-			$self->$key($params->{$key});
-		}
-	}
-	foreach my $key (sort { $a cmp $b } keys %$params) {
-		if ($key =~ /^_/) { # check for private fields
-			my $key_class = ref $params->{$key};
-			if ((defined $key_class) && ($key_class =~ /^JSON.*?Boolean$/smx)) {
-				$self->$key($params->{$key} ? 1 : 0);
-			} else {
-				$self->$key($params->{$key});
-			}
-		}
-	}
-	return $self;
-}
-
-sub AUTOLOAD {
-	my $self = shift;
-	my $type = ref($self) or Carp::croak "$self is not an object";
-
-	my $name = $Archive::Har::Entry::Timings::AUTOLOAD;
-	$name =~ s/.*://;   # strip fully-qualified portion
-
-	if ($name =~ /^[a-zA-Z]+$/) {
-		if (not exists $_fields{$name} ) {
-		    Carp::croak "Can't access '$name' field in class $type";
-		}
-		if (@_) {
-		    return $self->{$name} = shift;
-		} else {
-		    return $self->{$name};
-		}
-	} elsif ($name =~ /^_[a-zA-Z]+$/) { # private fields
-		if (@_) {
-		    return $self->{$name} = shift;
-		} else {
-	            return $self->{$name};
-		}
-	}
-	return;
-}
-
-sub TO_JSON {
-	my ($self) = @_;
-	my $json = {};
-	foreach my $key (sort { $a cmp $b } keys %$self) {
-		next unless (defined $self->{$key});
-		$json->{$key} = $self->{$key};
-	}
-	foreach my $mandatory_numeric (qw(send wait receive)) {
-		$json->{$mandatory_numeric} //= 0;
-		$json->{$mandatory_numeric} += 0;
-	}
-	return $json;
-}
-
-sub DESTROY {
-}
-
-1;
