@@ -5,114 +5,35 @@ use strict;
 use Carp();
 use Archive::Har::Page::PageTimings();
 
-=head1 NAME
-
-Archive::Har::Page - Represents a single page inside the HTTP Archive
-
-=head1 VERSION
-
-Version 0.06
-
-=cut
-
-our $VERSION = '0.06';
-
-=head1 SYNOPSIS
-
-    use Archive::Har();
-
-    my $http_archive_string = '"log": { "version": "1.1", .... ';
-    my $har = Archive::Har->new();
-    $har->string($http_archive_string);
-    foreach my $page ($har->pages()) {
-        print "DateTime: " . $page->startedDateTime() . "\n";
-        print "Id: " . $page->id() . "\n";
-        print "Title: ". $page->title() . "\n";
-        my $timing = $page->pageTimings();
-        print "Comment: " . $page->comment() . "\n";
-    }
-
-=head1 SUBROUTINES/METHODS
-
-=cut
-
-my (%_fields) = (
-			startedDateTime => undef,
-			id => undef,
-			title => undef,
-			pageTimings => undef,
-			comment => undef,
-		);
-
-=head2 startedDateTime
-
-returns the date and time stamp for the beginning of the page load (ISO 8601 format)
-
-=head2 id
-
-returns the unique identifier of a page within the Archive. This is referenced by the Archive::Har::Entry objects
-
-=head2 title
-
-returns the page title
-
-=head2 pageTimings
-
-returns the page timings object
-
-=head2 comment
-
-returns the comment about the Page
-
-=cut
+our $VERSION = 0.07;
 
 sub new {
 	my ($class, $params) = @_;
 	my $self = {};
 	bless $self, $class;
-	foreach my $key (sort { $a cmp $b } keys %_fields) {
-		if ($key eq 'pageTimings') {
-			$self->$key(Archive::Har::Page::PageTimings->new($params->{$key}));
-		} else {
-			my $key_class = ref $params->{$key};
-			if ((defined $key_class) && ($key_class =~ /^JSON.*?Boolean$/smx)) {
-				$self->$key($params->{$key} ? 1 : 0);
-			} else {
-				$self->$key($params->{$key});
-			}
+	if (defined $params) {
+		$self->id($params->{id});
+		if (defined $params->{title}) {
+			$self->title($params->{title});
 		}
-	}
-	foreach my $key (sort { $a cmp $b } keys %$params) {
-		if ($key =~ /^_/) { # check for private fields
-			my $key_class = ref $params->{$key};
-			if ((defined $key_class) && ($key_class =~ /^JSON.*?Boolean$/smx)) {
-				$self->$key($params->{$key} ? 1 : 0);
-			} else {
-				$self->$key($params->{$key});
+		if( defined $params->{startedDateTime}) {
+			$self->started_date_time($params->{startedDateTime});
+		}
+		$self->page_timings(Archive::Har::Page::PageTimings->new($params->{pageTimings} || {}));
+		if (defined $params->{comment}) {
+			$self->comment($params->{comment});
+		}
+		foreach my $key (sort { $a cmp $b } keys %{$params}) {
+			if ($key =~ /^_[[:alnum:]]+$/smx) { # private fields
+				if (defined $params->{$key}) {
+					$self->$key($params->{$key});
+				}
 			}
 		}
 	}
 	return $self;
 }
 
-sub startedDateTime {
-	my ($self, $new) = @_;
-	my $old = $self->{startedDateTime};
-	$old //= '0000-00-00T00:00:00.0+00:00';
-	if (@_ > 1) {
-		if (defined $new) {
-			if ($new =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+(?:\+\d{2}:\d{2}|Z)$/smx) {
-				$self->{startedDateTime} = $new;
-			} else {
-				Carp::croak("startedDateTime is not formatted correctly:$new");
-			}
-		} else {
-			$self->{startedDateTime} = undef;
-		}
-	}
-	return $old;
-}
-
 sub id {
 	my ($self, $new) = @_;
 	my $old = $self->{id};
@@ -125,50 +46,141 @@ sub id {
 sub title {
 	my ($self, $new) = @_;
 	my $old = $self->{title};
-	$old //= 'unknown';
 	if (@_ > 1) {
 		$self->{title} = $new;
 	}
+	if (defined $old) {
+		return $old;
+	} else {
+		return 'Unknown';
+	}
+	return $old;
+}
+
+sub started_date_time {
+	my ($self, $new) = @_;
+	my $old = $self->{startedDateTime};
+	if (@_ > 1) {
+		if (defined $new) {
+			if ($new =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+(?:\+\d{2}:\d{2}|Z)$/smx) {
+				$self->{startedDateTime} = $new;
+			} else {
+				Carp::croak("startedDateTime is not formatted correctly:$new");
+			}
+		} else {
+			$self->{startedDateTime} = undef;
+		}
+	}
+	if (defined $old) {
+		return $old;
+	} else {
+		return '0000-00-00T00:00:00.0+00:00';
+	}
+}
+
+sub page_timings {
+	my ($self, $new) = @_;
+	my $old = $self->{pageTimings};
+	if (@_ > 1) {
+		$self->{pageTimings} = $new;
+	}
+	return $old;
+}
+
+sub comment {
+	my ($self, $new) = @_;
+	my $old = $self->{comment};
+	if (@_ > 1) {
+		$self->{comment} = $new;
+	}
 	return $old;
 }
 
 sub AUTOLOAD {
-	my $self = shift;
-	my $type = ref($self) or Carp::croak "$self is not an object";
+	my ($self, $new) = @_;
+	my $type = ref $self or Carp::croak("$self is not an object");
 
 	my $name = $Archive::Har::Page::AUTOLOAD;
-	$name =~ s/.*://;   # strip fully-qualified portion
+	$name =~ s/.*://smx;   # strip fully-qualified portion
 
-	if ($name =~ /^[a-zA-Z]+$/) {
-		if (not exists $_fields{$name} ) {
-		    Carp::croak "Can't access '$name' field in class $type";
-		}
-		if (@_) {
-		    return $self->{$name} = shift;
-		} else {
-		    return $self->{$name};
-		}
-	} elsif ($name =~ /^_[a-zA-Z]+$/) { # private fields
-		if (@_) {
-		    return $self->{$name} = shift;
-		} else {
-	            return $self->{$name};
+	if ($name =~ /^_[[:alnum:]]+$/smx) { # private fields
+		my $old = $self->{$name};
+		if (@_ > 1) {
+			$self->{$name} = $new;
 		}
+		return $self->{$name};
+	} else {
+		Carp::croak("$name is not specified in the HAR 1.2 spec and does not start with an underscore");
 	}
 	return;
 }
 
 sub TO_JSON {
 	my ($self) = @_;
-	my $json = {};
-	foreach my $key (sort { $a cmp $b } keys %$self) {
-		next if (!defined $self->$key());
-		$json->{$key} = $self->$key();
+	my $json = {
+			title => $self->title(),
+			startedDateTime => $self->started_date_time(),
+			pageTimings => $self->page_timings(),
+		};
+	if (defined $self->id()) {
+		$json->{id} = $self->id();
+	}
+	if (defined $self->comment()) {
+		$json->{comment} = $self->comment();
+	}
+	foreach my $key (sort { $a cmp $b } keys %{$self}) {
+		next if (!defined $self->{$key});
+		if ($key =~ /^_[[:alnum:]]+$/smx) { # private fields
+			$json->{$key} = $self->{$key};
+		}
 	}
 	return $json;
 }
 
-sub DESTROY {
-}
-
 1;
+__END__
+=head1 NAME
+
+Archive::Har::Page - Represents a single page inside the HTTP Archive
+
+=head1 VERSION
+
+Version 0.07
+
+=head1 SYNOPSIS
+
+    use Archive::Har();
+
+    my $http_archive_string = '"log": { "version": "1.1", .... ';
+    my $har = Archive::Har->new();
+    $har->string($http_archive_string);
+    foreach my $page ($har->pages()) {
+        print "DateTime: " . $page->started_date_time() . "\n";
+        print "Id: " . $page->id() . "\n";
+        print "Title: ". $page->title() . "\n";
+        my $timing = $page->page_timings();
+        print "Comment: " . $page->comment() . "\n";
+    }
+
+=head1 SUBROUTINES/METHODS
+
+=head2 started_date_time
+
+returns the date and time stamp for the beginning of the page load (ISO 8601 format)
+
+=head2 id
+
+returns the unique identifier of a page within the Archive. This is referenced by the Archive::Har::Entry objects
+
+=head2 title
+
+returns the page title
+
+=head2 page_timings
+
+returns the page timings object
+
+=head2 comment
+
+returns the comment about the Page
+
